/**
 * Copyright (c) 2016-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#import <React/RCTUtils.h>
#import <Firebase/Firebase.h>

#import "FirebaseAnalyticsModule.h"

static NSString *const RNFBErrorDomain = @"RNFBErrorDomain";

@implementation FirebaseAnalytics {
    NSString *_userId;
}
#pragma mark -
#pragma mark Module Setup

  RCT_EXPORT_MODULE();

  + (BOOL)requiresMainQueueSetup
  {
      return YES;
  }

  - (instancetype)init
  {
      if (self = [super init]) {
          _userId = nil;
      }
      return self;
  }

  - (dispatch_queue_t)methodQueue {
    return dispatch_get_main_queue();
  }

#pragma mark -
#pragma mark Firebase Analytics Methods

  RCT_EXPORT_METHOD(logEvent:
    (NSString *) name
        params:
        (NSDictionary *) params
        resolver:
        (RCTPromiseResolveBlock) resolve
        rejecter:
        (RCTPromiseRejectBlock) reject) {
    @try {
        NSDictionary *eventParams = [self cleanJavascriptParams:params];
        [FIRAnalytics logEventWithName:name parameters:eventParams];
    } @catch (NSException *exception) {
      [self rejectPromiseWithExceptionDict:reject exception:exception];
    }

    resolve([NSNull null]);
  }

  RCT_EXPORT_METHOD(setAnalyticsCollectionEnabled:
    (BOOL) enabled
        resolver:
        (RCTPromiseResolveBlock) resolve
        rejecter:
        (RCTPromiseRejectBlock) reject) {
    @try {
      [FIRAnalytics setAnalyticsCollectionEnabled:enabled];
    } @catch (NSException *exception) {
      [self rejectPromiseWithExceptionDict:reject exception:exception];
    }

    resolve([NSNull null]);
  }

  RCT_EXPORT_METHOD(setUserId:
    (NSString *) id
        resolver:
        (RCTPromiseResolveBlock) resolve
        rejecter:
        (RCTPromiseRejectBlock) reject) {
    @try {
        _userId = id;
        [FIRAnalytics setUserID:id];
    } @catch (NSException *exception) {
        [self rejectPromiseWithExceptionDict:reject exception:exception];
    }
    resolve([NSNull null]);
  }

  RCT_EXPORT_METHOD(setUserProperty:
    (NSString *) name
        value:
        (NSString *) value
        resolver:
        (RCTPromiseResolveBlock) resolve
        rejecter:
        (RCTPromiseRejectBlock) reject) {
    @try {
      [FIRAnalytics setUserPropertyString:value forName:name];
    } @catch (NSException *exception) {
      [self rejectPromiseWithExceptionDict:reject exception:exception];
    }
    resolve([NSNull null]);
  }

  RCT_EXPORT_METHOD(setUserProperties:
    (NSDictionary *) properties
        resolver:
        (RCTPromiseResolveBlock) resolve
        rejecter:
        (RCTPromiseRejectBlock) reject) {
    @try {
      [properties enumerateKeysAndObjectsUsingBlock:^(id key, id value, BOOL *stop) {
        [FIRAnalytics setUserPropertyString:value forName:key];
      }];
    } @catch (NSException *exception) {
      [self rejectPromiseWithExceptionDict:reject exception:exception];
    }
    resolve([NSNull null]);
  }

  RCT_EXPORT_METHOD(resetAnalyticsData:
    (RCTPromiseResolveBlock) resolve
        rejecter:
        (RCTPromiseRejectBlock) reject) {
    @try {
      [FIRAnalytics resetAnalyticsData];
    } @catch (NSException *exception) {
      [self rejectPromiseWithExceptionDict:reject exception:exception];
    }
    resolve([NSNull null]);
  }

  RCT_EXPORT_METHOD(setSessionTimeoutDuration:
    (double) milliseconds
        resolver:
        (RCTPromiseResolveBlock) resolve
        rejecter:
        (RCTPromiseRejectBlock) reject) {
    // Do nothing - this only exists in android
    resolve([NSNull null]);
  }

  RCT_EXPORT_METHOD(getAppInstanceId:
  (RCTPromiseResolveBlock) resolve
        rejecter:
        (RCTPromiseRejectBlock) reject) {
      resolve([FIRAnalytics appInstanceID]);
  }

  RCT_EXPORT_METHOD(setDefaultEventParameters:
        (NSDictionary *) params
        resolver:
        (RCTPromiseResolveBlock) resolve
        rejecter:
        (RCTPromiseRejectBlock) reject) {
    @try {
        [FIRAnalytics setDefaultEventParameters:[self cleanJavascriptParams:params]];
    } @catch (NSException *exception) {
      [self rejectPromiseWithExceptionDict:reject exception:exception];
    }

    resolve([NSNull null]);
  }

RCT_EXPORT_METHOD(getInstance:
      (RCTPromiseResolveBlock) resolve
      rejecter:
      (RCTPromiseRejectBlock) reject) {
    resolve(FIRAnalytics.appInstanceID);
}

#pragma mark -
#pragma mark Private methods

  - (NSDictionary *)cleanJavascriptParams:(NSDictionary *)params {
    NSMutableDictionary *newParams = [params mutableCopy];
    if (newParams[kFIRParameterItems]) {
      NSMutableArray *newItems = [NSMutableArray array];
      [(NSArray *)newParams[kFIRParameterItems] enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        NSMutableDictionary *item = [obj mutableCopy];
        if (item[kFIRParameterQuantity]) {
          item[kFIRParameterQuantity] = @([item[kFIRParameterQuantity] integerValue]);
        }
        [newItems addObject:[item copy]];
      }];
      newParams[kFIRParameterItems] = [newItems copy];
    }
    return [newParams copy];
  }

  - (void)rejectPromiseWithExceptionDict:(RCTPromiseRejectBlock)reject exception:(NSException *)exception {
    NSMutableDictionary *userInfo = [NSMutableDictionary dictionary];

    [userInfo setValue:@(YES) forKey:@"fatal"];
    [userInfo setValue:@"unknown" forKey:@"code"];
    [userInfo setValue:exception.reason forKey:@"message"];
    [userInfo setValue:exception.name forKey:@"nativeErrorCode"];
    [userInfo setValue:exception.reason forKey:@"nativeErrorMessage"];

    NSError *error = [NSError errorWithDomain:RNFBErrorDomain code:666 userInfo:userInfo];

    reject(exception.name, exception.reason, error);
  }

@end
